import { Injectable, Inject, ɵɵdefineInjectable, ɵɵinject, Optional, InjectionToken, SkipSelf, inject, Component, ChangeDetectionStrategy, ViewEncapsulation, ElementRef, Input, NgModule } from '@angular/core';
import { DOCUMENT } from '@angular/common';
import { TRANSLOCO_TRANSPILER, getPipeValue, TranslocoService, TRANSLOCO_MISSING_HANDLER, TRANSLOCO_SCOPE, TRANSLOCO_LANG } from '@ngneat/transloco';
import { BehaviorSubject, Subscription, of, concat, EMPTY, combineLatest } from 'rxjs';
import { map, first, shareReplay, switchMap, skip, distinctUntilChanged } from 'rxjs/operators';

/**
 * @fileoverview added by tsickle
 * Generated from: src/models/external-link.model.ts
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Model used to describe a link that references a resource outside the application.
 * @record
 */
function ExternalLink() { }
if (false) {
    /**
     * Url of the resource.
     * @type {?}
     */
    ExternalLink.prototype.url;
    /**
     * Browser target in wich the link should be opened.
     * @type {?|undefined}
     */
    ExternalLink.prototype.target;
}
/**
 * Checks whether the specified value conforms to the `ExternalLink` interface.
 *
 * @param {?} value Value that is to be checked.
 * @return {?} `true` if the specified value conforms to the `ExternalLink` interface, `false` if not.
 */
function isExternalLinkObject(value) {
    if (typeof value !== 'object' || value === null) {
        return false;
    }
    /** @type {?} */
    const url = ((/** @type {?} */ (value))).url;
    /** @type {?} */
    const target = ((/** @type {?} */ (value))).target;
    return typeof url === 'string' && (target === undefined || target === null || typeof target === 'string');
}

/**
 * @fileoverview added by tsickle
 * Generated from: src/transpilers/block-transpiler.ts
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Base class for transpilers that can transpile contents of a block that is bounded by specific start and end tokens.
 * @abstract
 */
class BlockTranspiler {
    /**
     * Creates a new `BlockTranspiler` for the specified boundary tokens.
     * @param {?} blockStartToken
     * @param {?} blockEndToken
     */
    constructor(blockStartToken, blockEndToken) {
        this.blockStartToken = blockStartToken;
        this.blockEndToken = blockEndToken;
    }
    /**
     * \@inheritdoc
     * @param {?} translation
     * @param {?} offset
     * @return {?}
     */
    tokenize(translation, offset) {
        /**
         * @param {?} token
         * @return {?}
         */
        function recognize(token) {
            return (translation.substring(offset, offset + token.length) === token
                ? { nextOffset: offset + token.length, token: new BlockBoundary(token) }
                : undefined);
        }
        return recognize(this.blockStartToken) || recognize(this.blockEndToken);
    }
    /**
     * \@inheritdoc
     * @param {?} start
     * @param {?} context
     * @return {?}
     */
    transpile(start, context) {
        /** @type {?} */
        const nextToken = context.tokens[start];
        if (!this.isStartTag(nextToken)) {
            return undefined;
        }
        const { nextOffset, renderers } = context.transpileUntil(start + 1, (/**
         * @param {?} token
         * @return {?}
         */
        (token) => this.isEndTag(token)));
        return {
            nextOffset: Math.min(nextOffset + 1, context.tokens.length),
            renderer: this.createRenderer(renderers)
        };
    }
    /**
     * Checks whether the specified token represents the start boundary of a block that can be transpiled.
     *
     * @private
     * @param {?} token Token which is to be checked.
     * @return {?} `true` if the token is a start boundary of the block.
     */
    isStartTag(token) {
        return token instanceof BlockBoundary && token.token === this.blockStartToken;
    }
    /**
     * Checks whether the specified token represents the end boundary of a block that can be transpiled.
     *
     * @private
     * @param {?} token Token which is to be checked.
     * @return {?} `true` if the token is a end boundary of the block.
     */
    isEndTag(token) {
        return token instanceof BlockBoundary && token.token === this.blockEndToken;
    }
}
if (false) {
    /**
     * String that defines the start token for the block.
     * @type {?}
     * @private
     */
    BlockTranspiler.prototype.blockStartToken;
    /**
     * String that defines the end token for the block.
     * @type {?}
     * @private
     */
    BlockTranspiler.prototype.blockEndToken;
    /**
     * Creates the block rendering function for the given transpiled contents of the block.
     *
     * @abstract
     * @protected
     * @param {?} childRenderers Contents of the block that was transpiled to a sequence of child renderers.
     * @return {?} A renderer that combines the given child renderers, representing the contents of the block, into a single
     *                         rendering function.
     */
    BlockTranspiler.prototype.createRenderer = function (childRenderers) { };
}
/**
 * Class used for representing the boundary tokens of a block.
 */
class BlockBoundary {
    /**
     * @param {?} token
     */
    constructor(token) {
        this.token = token;
    }
}
if (false) {
    /** @type {?} */
    BlockBoundary.prototype.token;
}

/**
 * @fileoverview added by tsickle
 * Generated from: src/translation-markup-renderer-factory.ts
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Factory used for creating different types of `TranslationMarkupRenderer` instances.
 */
class TranslationMarkupRendererFactory {
    /**
     * Creates a new `TranslationMarkupRendererFactory` instance.
     *
     * @param {?} document Document instance used by the renderers to create the actual DOM nodes.
     */
    constructor(document) {
        this.document = document;
    }
    /**
     * Creates a markup renderer to render text. This can be either a static text or on that dynamically resolved based on the translation
     * parameters.
     *
     * @param {?} text Text that is to be rendered or a function that returns the text to be rendered based on the translation parameters.
     * @return {?} A `TranslationMarkupRenderer` that renders the specified text.
     */
    createTextRenderer(text) {
        /** @type {?} */
        const document = this.document;
        return (/**
         * @param {?} translationParameters
         * @return {?}
         */
        function renderText(translationParameters) {
            return document.createTextNode(typeof text === 'string' ? text : text(translationParameters));
        });
    }
    /**
     * @param {?} elementTag
     * @param {?=} childRenderers
     * @return {?}
     */
    createElementRenderer(elementTag, childRenderers) {
        /** @type {?} */
        const document = this.document;
        return (/**
         * @param {?} translationParameters
         * @return {?}
         */
        function renderElement(translationParameters) {
            /** @type {?} */
            const element = document.createElement(elementTag);
            for (const renderChild of childRenderers || []) {
                element.appendChild(renderChild(translationParameters));
            }
            return element;
        });
    }
}
TranslationMarkupRendererFactory.decorators = [
    { type: Injectable, args: [{ providedIn: 'root' },] }
];
/** @nocollapse */
TranslationMarkupRendererFactory.ctorParameters = () => [
    { type: undefined, decorators: [{ type: Inject, args: [DOCUMENT,] }] }
];
/** @nocollapse */ TranslationMarkupRendererFactory.ɵprov = ɵɵdefineInjectable({ factory: function TranslationMarkupRendererFactory_Factory() { return new TranslationMarkupRendererFactory(ɵɵinject(DOCUMENT)); }, token: TranslationMarkupRendererFactory, providedIn: "root" });
if (false) {
    /**
     * Document instance used by the renderers to create the actual DOM nodes.
     * @type {?}
     * @private
     */
    TranslationMarkupRendererFactory.prototype.document;
}

/**
 * @fileoverview added by tsickle
 * Generated from: src/transpilers/bold-text-transpiler.ts
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Transpiler that parses bold tags (`[b]...[/b]`) and creates a renderer that wraps the content in an HTML bold element (`<b>...</b>`).
 */
class BoldTextTranspiler extends BlockTranspiler {
    /**
     * Creates a `BoldTextTranspiler` instance that uses the specified renderer factory.
     * @param {?} rendererFactory
     */
    constructor(rendererFactory) {
        super('[b]', '[/b]');
        this.rendererFactory = rendererFactory;
    }
    /**
     * \@inheritdoc
     * @protected
     * @param {?} childRenderers
     * @return {?}
     */
    createRenderer(childRenderers) {
        return this.rendererFactory.createElementRenderer('b', childRenderers);
    }
}
BoldTextTranspiler.decorators = [
    { type: Injectable }
];
/** @nocollapse */
BoldTextTranspiler.ctorParameters = () => [
    { type: TranslationMarkupRendererFactory }
];
if (false) {
    /**
     * Renderer factory which will be used for creating the HTML bold element renderer.
     * @type {?}
     * @private
     */
    BoldTextTranspiler.prototype.rendererFactory;
}

/**
 * @fileoverview added by tsickle
 * Generated from: src/transpilers/contextual-link-block-transpiler.ts
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Transpiler for named (special start and end token) link blocks. This can be used instead of the more generic `LinkTranspiler` to create
 * context specific link block transpilers.
 */
class ContextualLinkBlockTranspiler extends BlockTranspiler {
    /**
     * Creates a new `NamedLinkBlockTranspiler` for the specified start and end token.
     * @param {?} linkStartToken
     * @param {?} linkEndToken
     * @param {?} resolveLinkSpecification
     * @param {?} rendererFactory
     * @param {?} linkRenderers
     */
    constructor(
    /** Token that defined the start of the link. */
    linkStartToken, 
    /** Token that defined the end of the link. */
    linkEndToken, resolveLinkSpecification, rendererFactory, linkRenderers) {
        super(linkStartToken, linkEndToken);
        this.resolveLinkSpecification = resolveLinkSpecification;
        this.rendererFactory = rendererFactory;
        this.linkRenderers = linkRenderers;
    }
    /**
     * \@inheritdoc
     * @param {?} childRenderers
     * @return {?}
     */
    createRenderer(childRenderers) {
        /** @type {?} */
        const anchorRenderer = this.rendererFactory.createElementRenderer('a', childRenderers);
        /** @type {?} */
        const findLinkRenderer = (/**
         * @param {?} link
         * @return {?}
         */
        (link) => this.linkRenderers.find((/**
         * @param {?} linkRenderer
         * @return {?}
         */
        (linkRenderer) => linkRenderer.supports(link))));
        /** @type {?} */
        const resolveLinkSpecification = this.resolveLinkSpecification;
        return (/**
         * @param {?} translationParameters
         * @return {?}
         */
        function namedLinkRenderer(translationParameters) {
            /** @type {?} */
            const anchorElement = anchorRenderer(translationParameters);
            /** @type {?} */
            const link = ('static' in resolveLinkSpecification ? resolveLinkSpecification.static :
                'parameterKey' in resolveLinkSpecification ? translationParameters[resolveLinkSpecification.parameterKey] :
                    resolveLinkSpecification.resolve(translationParameters));
            /** @type {?} */
            const linkRenderer = findLinkRenderer(link);
            if (linkRenderer) {
                linkRenderer.render(link, anchorElement);
            }
            return anchorElement;
        });
    }
}
if (false) {
    /**
     * Object that specifies how to resolve the link.
     * @type {?}
     * @private
     */
    ContextualLinkBlockTranspiler.prototype.resolveLinkSpecification;
    /**
     * Renderer factory used to create the renderer for the HTML anchor element that will be substituted in place of the token.
     * @type {?}
     * @private
     */
    ContextualLinkBlockTranspiler.prototype.rendererFactory;
    /**
     * Set of link renderers that are used to apply the link model from the translation parameter to the anchor element.
     * @type {?}
     * @private
     */
    ContextualLinkBlockTranspiler.prototype.linkRenderers;
}

/**
 * @fileoverview added by tsickle
 * Generated from: src/transpilers/substitution-transpiler.ts
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Transpiler that substitutes a fixed token. A subclass is responsible for rendering the actual markup that will be used instead of the
 * token. This is done by providing an implementation for the abstract `createRenderer` function.
 * @abstract
 */
class SubstitutionTranspiler {
    /**
     * Creates a new `SubstitutionTranspiler` for the specified substitution token.
     * @param {?} substitutionToken
     */
    constructor(substitutionToken) {
        this.substitutionToken = substitutionToken;
        /**
         * Object that holds the value of lazily initialized property.
         */
        this.deferred = {};
    }
    /**
     * \@inheritdoc
     * @param {?} translation
     * @param {?} offset
     * @return {?}
     */
    tokenize(translation, offset) {
        if (translation.startsWith(this.substitutionToken, offset)) {
            return {
                nextOffset: offset + this.substitutionToken.length,
                token: new SubstitutionToken(this.substitutionToken)
            };
        }
        return undefined;
    }
    /**
     * \@inheritdoc
     * @param {?} offset
     * @param {?} __1
     * @return {?}
     */
    transpile(offset, { tokens }) {
        /** @type {?} */
        const nextToken = tokens[offset];
        if (!(nextToken instanceof SubstitutionToken) || nextToken.token !== this.substitutionToken) {
            return undefined;
        }
        return {
            nextOffset: offset + 1,
            renderer: this.renderer
        };
    }
    /**
     * Cached value of the result of the `createRenderer` function.
     * @private
     * @return {?}
     */
    get renderer() {
        return this.deferred.renderer || (this.deferred.renderer = this.createRenderer());
    }
}
if (false) {
    /**
     * Object that holds the value of lazily initialized property.
     * @type {?}
     * @private
     */
    SubstitutionTranspiler.prototype.deferred;
    /**
     * String representing the token that will be substituted with the output of the resulting renderer.
     * @type {?}
     * @private
     */
    SubstitutionTranspiler.prototype.substitutionToken;
    /**
     * Creates the renderer responsible for generating the markup that will be substituted in place of the substitution token.
     * @abstract
     * @protected
     * @return {?}
     */
    SubstitutionTranspiler.prototype.createRenderer = function () { };
}
/**
 * Class for representing tokens that are to be replaced with the output of a specific markup renderer.
 */
class SubstitutionToken {
    /**
     *  Creates a new `SubstitutionToken` instance for the specified token.
     * @param {?} token
     */
    constructor(token) {
        this.token = token;
    }
}
if (false) {
    /** @type {?} */
    SubstitutionToken.prototype.token;
}

/**
 * @fileoverview added by tsickle
 * Generated from: src/transpilers/contextual-link-substitution-transpiler.ts
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Configuration options for the `ContextualLinkSubstitutionTranspiler`.
 * @record
 */
function ContextualLinkSubstitutionTranspilerOptions() { }
if (false) {
    /**
     * Property that defines how to resolve the label text of the link that is to be rendered. The label can be resolved in three different
     * ways:
     *
     * - A static label: `{ static: 'label text' }`
     * - A value stored in the translation parameters: `{ parameterKey: 'nameOfTheParameterContainingTheLabelText' }`
     * - A dynamically resolved label using a function that receives the translation parameters as input:
     *   `{ resolve: (params) => params.foo + '-' + params.bar + '-' + Date.now() }`
     * @type {?}
     */
    ContextualLinkSubstitutionTranspilerOptions.prototype.label;
    /**
     * A resolver for the link model specifiying the target location. The link model can be resolved in the same three ways as the label:
     *
     * - A static link: `{ static: { url: 'https://example.com/', target: '_self' } }`
     * - A link stored in the translation parameters: `{ parameterKey: 'targetLink' }`
     * - A dynamically resolved link using a function that receives the translation parameters as input:
     *   `{ resolve: (params) => 'https://petstore.com/' + params.petType }`
     * @type {?}
     */
    ContextualLinkSubstitutionTranspilerOptions.prototype.link;
}
/**
 * A substitution transpiler that replaces a specific token with a link. This transpiler is quite useful for defining tokens that are only
 * valid within a specific context (i.e. a limited set of translation entries).
 */
class ContextualLinkSubstitutionTranspiler extends SubstitutionTranspiler {
    /**
     * Creates a new `ContextualLinkSubstitutionTranspiler` for the specified token and which uses the provided options.
     * @param {?} token
     * @param {?} options
     * @param {?} rendererFactory
     * @param {?} linkRenderers
     */
    constructor(
    /** The token which will be substituted with a link. */
    token, options, rendererFactory, linkRenderers) {
        super(token);
        this.options = options;
        this.rendererFactory = rendererFactory;
        this.linkRenderers = linkRenderers;
    }
    /**
     * \@inheritdoc
     * @protected
     * @return {?}
     */
    createRenderer() {
        /** @type {?} */
        const labelRenderer = this.rendererFactory.createTextRenderer(((/**
         * @return {?}
         */
        () => {
            const { label } = this.options;
            if ('static' in label) {
                return label.static;
            }
            if ('resolve' in label) {
                return label.resolve; // tslint:disable-line:no-unbound-method
            }
            return (/**
             * @param {?} translationParams
             * @return {?}
             */
            (translationParams) => String(translationParams[label.parameterKey] || ''));
        }))());
        /** @type {?} */
        const anchorRenderer = this.rendererFactory.createElementRenderer('a', [labelRenderer]);
        /** @type {?} */
        const options = this.options;
        /** @type {?} */
        const findLinkRenderer = (/**
         * @param {?} link
         * @return {?}
         */
        (link) => this.linkRenderers.find((/**
         * @param {?} linkRenderer
         * @return {?}
         */
        (linkRenderer) => linkRenderer.supports(link))));
        return (/**
         * @param {?} translationParameters
         * @return {?}
         */
        function renderSubstitutionLink(translationParameters) {
            /** @type {?} */
            const anchorElement = anchorRenderer(translationParameters);
            /** @type {?} */
            const link = ('static' in options.link ? options.link.static :
                'parameterKey' in options.link ? translationParameters[options.link.parameterKey] :
                    options.link.resolve(translationParameters));
            /** @type {?} */
            const linkRenderer = findLinkRenderer(link);
            if (linkRenderer) {
                linkRenderer.render(link, anchorElement);
            }
            return anchorElement;
        });
    }
}
if (false) {
    /**
     * Options that define how to resolve the label text and link model.
     * @type {?}
     * @private
     */
    ContextualLinkSubstitutionTranspiler.prototype.options;
    /**
     * Renderer factory used to create the renderer for the HTML anchor element that will be substituted in place of the token.
     * @type {?}
     * @private
     */
    ContextualLinkSubstitutionTranspiler.prototype.rendererFactory;
    /**
     * Set of link renderers that are used to apply the link model from the translation parameter to the anchor element.
     * @type {?}
     * @private
     */
    ContextualLinkSubstitutionTranspiler.prototype.linkRenderers;
}

/**
 * @fileoverview added by tsickle
 * Generated from: src/transpilers/italic-text-transpiler.ts
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Transpiler that parses italic tags (`[i]...[/i]`) and creates a renderer that wraps the content in an HTML italic element (`<i>...</i>`).
 */
class ItalicTextTranspiler extends BlockTranspiler {
    /**
     * Creates an `ItalicTextTranspiler` instance that uses the specified renderer factory.
     * @param {?} rendererFactory
     */
    constructor(rendererFactory) {
        super('[i]', '[/i]');
        this.rendererFactory = rendererFactory;
    }
    /**
     * \@inheritdoc
     * @protected
     * @param {?} childRenderers
     * @return {?}
     */
    createRenderer(childRenderers) {
        return this.rendererFactory.createElementRenderer('i', childRenderers);
    }
}
ItalicTextTranspiler.decorators = [
    { type: Injectable }
];
/** @nocollapse */
ItalicTextTranspiler.ctorParameters = () => [
    { type: TranslationMarkupRendererFactory }
];
if (false) {
    /**
     * Renderer factory which will be used for creating the HTML italic element renderer.
     * @type {?}
     * @private
     */
    ItalicTextTranspiler.prototype.rendererFactory;
}

/**
 * @fileoverview added by tsickle
 * Generated from: src/utils/array.ts
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Wraps the specified value in an array unless the value already is an array.
 *
 * @template T
 * @param {?} value Value that is to be wrapped in an array.
 * @return {?} If the specified value is an array the specified value is returned as is, otherwise it will be wrapped in array,
 *                containing just the value.
 */
function asArray(value) {
    return Array.isArray(value) ? value : [value];
}
/**
 * Flattens a recursive array (of arbitrary depth) into a one dimensional array. If the specified value is just a scalar value, then it will
 * be wrapped in an array containing only that value.
 *
 * @template T
 * @param {?} value Value which is to be converted to a flattened array.
 * @return {?} A one dimensional array containing all elements from the recursive array.
 */
function asFlatArray(value) {
    return Array.isArray(value)
        ? value.reduce((/**
         * @param {?} partialResult
         * @param {?} element
         * @return {?}
         */
        (partialResult, element) => partialResult.concat(asFlatArray(element))), [])
        : [value];
}

/**
 * @fileoverview added by tsickle
 * Generated from: src/utils/provider-binding.ts
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Definition (without token binding) for providers that use the specified value for injection.
 * @record
 * @template T
 */
function UnboundValueProvider() { }
if (false) {
    /**
     * The value to inject.
     * @type {?}
     */
    UnboundValueProvider.prototype.useValue;
}
/**
 * Definition (without token binding) for providers that instantiate a specific class for injection.
 * @record
 * @template T
 */
function UnboundClassProvider() { }
if (false) {
    /**
     * Class to instantiate when being injected for a specific token.
     * @type {?}
     */
    UnboundClassProvider.prototype.useClass;
}
/**
 * Definition (without token binding) for providers that reference another token to use for injection.
 * @record
 * @template T
 */
function UnboundExistingProvider() { }
if (false) {
    /**
     * Existing token to return (equivalent to `injector.get(useExisting)`).
     * @type {?}
     */
    UnboundExistingProvider.prototype.useExisting;
}
/**
 * Definition (without token binding) for providers that use a factory function to create injection values.
 * @record
 * @template T
 */
function UnboundFactoryProvider() { }
if (false) {
    /**
     * A list of tokens which need to be resolved by the injector. The list of values is then used as arguments to the `useFactory`
     * function.
     * @type {?|undefined}
     */
    UnboundFactoryProvider.prototype.deps;
    /**
     * A function to invoke to create a value when this provider is injected for a specific token. The function is invoked with resolved
     * values of tokens in the `deps` field.
     * @param {...?} deps
     * @return {?}
     */
    UnboundFactoryProvider.prototype.useFactory = function (deps) { };
}
/**
 * Extra binding options for the `bindProvider` function.
 * @record
 * @template U
 */
function BindProviderOptions() { }
if (false) {
    /**
     * Whether the provider should be contribute to a "multi-provider" (resulting an array of instances when injected).
     * @type {?|undefined}
     */
    BindProviderOptions.prototype.multi;
    /**
     * Default provider definition to use when the provider definition passed to the `bindProvider` function is `undefined`.
     * @type {?|undefined}
     */
    BindProviderOptions.prototype.default;
}
/**
 * Binds the given provider definition to the specified token and returns a `Provider` entry that can be used in the providers list of
 * Angular module definitions.
 *
 * @template T, U
 * @param {?} token           Token which for which the provider is to be defined.
 * @param {?} unboundProvider Definition of a provider which should be bound to the specified token.
 * @param {?=} options         Optional extra binding options.
 * @return {?}
 */
function bindProvider(token, unboundProvider, options = {}) {
    return (unboundProvider ? (((/** @type {?} */ (unboundProvider))).apply ?
        [
            unboundProvider,
            {
                provide: token,
                useExisting: (/** @type {?} */ (unboundProvider)),
                multi: options.multi
            }
        ] :
        ((/** @type {?} */ (unboundProvider))).useValue ?
            {
                provide: token,
                useValue: ((/** @type {?} */ (unboundProvider))).useValue,
                multi: options.multi
            } :
            ((/** @type {?} */ (unboundProvider))).useClass ?
                {
                    provide: token,
                    useClass: ((/** @type {?} */ (unboundProvider))).useClass,
                    multi: options.multi
                } :
                ((/** @type {?} */ (unboundProvider))).useExisting ?
                    {
                        provide: token,
                        useExisting: ((/** @type {?} */ (unboundProvider))).useExisting,
                        multi: options.multi
                    } :
                    ((/** @type {?} */ (unboundProvider))).useFactory ?
                        {
                            provide: token,
                            useFactory: ((/** @type {?} */ (unboundProvider))).useFactory,
                            // tslint:disable-line:no-unbound-method
                            deps: ((/** @type {?} */ (unboundProvider))).deps,
                            multi: options.multi
                        } :
                        bindProviderError(`Invalid unbound provider was specified for "${token}"`)) :
        options.default ? (((/** @type {?} */ (options.default))).apply ?
            [
                options.default,
                {
                    provide: token,
                    useExisting: (/** @type {?} */ (options.default)),
                    multi: options.multi
                }
            ] :
            ((/** @type {?} */ (options.default))).useValue ?
                {
                    provide: token,
                    useValue: ((/** @type {?} */ (options.default))).useValue,
                    multi: options.multi
                } :
                ((/** @type {?} */ (options.default))).useClass ?
                    {
                        provide: token,
                        useClass: ((/** @type {?} */ (options.default))).useClass,
                        multi: options.multi
                    } :
                    ((/** @type {?} */ (options.default))).useExisting ?
                        {
                            provide: token,
                            useExisting: ((/** @type {?} */ (options.default))).useExisting,
                            multi: options.multi
                        } :
                        ((/** @type {?} */ (options.default))).useFactory ?
                            {
                                provide: token,
                                useFactory: ((/** @type {?} */ (options.default))).useFactory,
                                // tslint:disable-line:no-unbound-method
                                deps: ((/** @type {?} */ (options.default))).deps,
                                multi: options.multi
                            } :
                            bindProviderError(`Invalid default unbound provider was specified for "${token}"`)) :
            []);
}
class BindProviderError extends Error {
}
/**
 * @param {?} message
 * @return {?}
 */
function bindProviderError(message) {
    throw new BindProviderError(message);
}

/**
 * @fileoverview added by tsickle
 * Generated from: src/link-renderer.model.ts
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Interface for objects that can render a link. Note that this is abstract class instead of an interface so the class itself can be used as
 * token for injection of the link renderers.
 * @abstract
 * @template T
 */
class LinkRenderer {
}
if (false) {
    /**
     * Checks whether the link renderer supports the specified link.
     *
     * @abstract
     * @param {?} link Link value for which is to be checked if the renderer supports it.
     * @return {?} `true` if the specified link is supported and it safe to be used as argument for the `render` function, `false` if the
     *               link is not supported by the renderer.
     */
    LinkRenderer.prototype.supports = function (link) { };
    /**
     * Renders the given link to the specified target element.
     *
     * @abstract
     * @param {?} link          Link that is to be rendered.
     * @param {?} targetElement Target anchor element to which the link should be rendered.
     * @return {?}
     */
    LinkRenderer.prototype.render = function (link, targetElement) { };
}
/**
 * Creates a provider definition for the specified link renderer. The resulting `Provider` object can be used to make the renderer available
 * within a module or component injector scope.
 *
 * @template T
 * @param {?} linkRenderer Specification of the link renderer for which the provider definition is to be created.
 * @return {?}
 */
function provideLinkRenderer(linkRenderer) {
    return bindProvider(LinkRenderer, linkRenderer, { multi: true });
}

/**
 * @fileoverview added by tsickle
 * Generated from: src/transpilers/link-transpiler.ts
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * A transpiler that can be used to parse links in a translation value and convert those to a renderer that generates an HTML anchor
 * element. Supports the following syntax: `[link:parameterKey]...[/link]`, where `parameterKey` should be substituted with the actual key
 * of the parameter that stores the target link. Contents between the opening and closing tags will be recursively transpiled.
 */
class LinkTranspiler {
    /**
     * Creates a new `LinkTranspiler` that uses the specified renderer factory for creating the HTML anchor element (`<a>`) and which
     * delegates the rendering of link models to the given set of link renderers.
     * @param {?} rendererFactory
     * @param {?} linkRenderers
     */
    constructor(rendererFactory, 
    /** Set of link renderers that are used to apply the link model from the translation parameter to the anchor element. */
    linkRenderers) {
        this.rendererFactory = rendererFactory;
        this.linkRenderers = !linkRenderers ? [] : asArray(linkRenderers);
    }
    /**
     * \@inheritdoc
     * @param {?} translation
     * @param {?} offset
     * @return {?}
     */
    tokenize(translation, offset) {
        if (translation.startsWith(LINK_END_TOKEN, offset)) {
            return {
                nextOffset: offset + LINK_END_TOKEN.length,
                token: LINK_END
            };
        }
        if (!translation.startsWith(LINK_START_TOKEN, offset)) {
            return undefined;
        }
        /** @type {?} */
        const end = translation.indexOf(']', offset + LINK_START_TOKEN.length);
        if (end < 0) {
            return undefined;
        }
        /** @type {?} */
        const parameterKey = translation.substring(offset + LINK_START_TOKEN.length, end);
        return {
            nextOffset: end + 1,
            token: new LinkStart(parameterKey)
        };
    }
    /**
     * \@inheritdoc
     * @param {?} start
     * @param {?} context
     * @return {?}
     */
    transpile(start, context) {
        /** @type {?} */
        const nextToken = context.tokens[start];
        if (!(nextToken instanceof LinkStart)) {
            return undefined;
        }
        const { nextOffset, renderers } = context.transpileUntil(start + 1, (/**
         * @param {?} token
         * @return {?}
         */
        (token) => token === LINK_END));
        return {
            nextOffset: Math.min(nextOffset + 1, context.tokens.length),
            renderer: this.createRenderer(nextToken.parameterKey, renderers)
        };
    }
    /**
     * Creates a renderer that generates an HTML anchor tag and applies the link model stored in the specified translation parameter.
     *
     * @protected
     * @param {?} parameterKey   Key that identifies the translation parameter containing the link model.
     * @param {?} childRenderers Transpiled content of the link.
     * @return {?} A markup renderer that creates an HTML representation of the link.
     */
    createRenderer(parameterKey, childRenderers) {
        /** @type {?} */
        const anchorRenderer = this.rendererFactory.createElementRenderer('a', childRenderers);
        /** @type {?} */
        const findLinkRenderer = (/**
         * @param {?} link
         * @return {?}
         */
        (link) => this.linkRenderers.find((/**
         * @param {?} linkRenderer
         * @return {?}
         */
        (linkRenderer) => linkRenderer.supports(link))));
        /**
         * @param {?} translationParameters
         * @return {?}
         */
        function renderLink(translationParameters) {
            /** @type {?} */
            const anchorElement = anchorRenderer(translationParameters);
            /** @type {?} */
            const link = translationParameters[parameterKey];
            /** @type {?} */
            const linkRenderer = findLinkRenderer(link);
            if (linkRenderer) {
                linkRenderer.render(link, anchorElement);
            }
            return anchorElement;
        }
        return renderLink;
    }
}
LinkTranspiler.decorators = [
    { type: Injectable }
];
/** @nocollapse */
LinkTranspiler.ctorParameters = () => [
    { type: TranslationMarkupRendererFactory },
    { type: undefined, decorators: [{ type: Inject, args: [LinkRenderer,] }, { type: Optional }] }
];
if (false) {
    /**
     * Set of link renderers that are used to apply the link model from the translation parameter to the anchor element.
     * @type {?}
     * @private
     */
    LinkTranspiler.prototype.linkRenderers;
    /**
     * Renderer factory which will be used for creating the HTML anchor element renderer.
     * @type {?}
     * @private
     */
    LinkTranspiler.prototype.rendererFactory;
}
/**
 * Prefix of a link start token.
 * @type {?}
 */
const LINK_START_TOKEN = '[link:';
/**
 * String that defines the end tag of a link.
 * @type {?}
 */
const LINK_END_TOKEN = '[/link]';
/**
 * Token that represents the closing tag of a link.
 * @type {?}
 */
const LINK_END = new (class LinkEnd {
})();
/**
 * Class used for modeling the link start tokens.
 */
class LinkStart {
    /**
     * Creates a new `LinkStart` that references the specified parameter.
     * @param {?} parameterKey
     */
    constructor(parameterKey) {
        this.parameterKey = parameterKey;
    }
}
if (false) {
    /**
     * Key of the parameter which stores the link model.
     * @type {?}
     */
    LinkStart.prototype.parameterKey;
}

/**
 * @fileoverview added by tsickle
 * Generated from: src/transpilers/string-interpolation-transpiler.ts
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Interface for objects that can detect interpolation expressions.
 * @record
 */
function InterpolationExpressionMatcher() { }
if (false) {
    /**
     * Checks whether an interpolation expression is found in the given string at the specified offset.
     *
     * @param {?} source String where the interpolation expression might be found.
     * @param {?} offset Position within the string which should be checked for the presence of an interpolation expression.
     * @return {?} Length of the interpolation expression in number of characters or `undefined` if no expression was found.
     */
    InterpolationExpressionMatcher.prototype.matchExpression = function (source, offset) { };
}
/**
 * Interpolation expression matcher for the default syntax of Transloco interpolation expressions: `{{...}}`.
 *
 * @param {?} translationValue String where the interpolation expression might be found.
 * @param {?} offset           Position within the string which should be checked for the presence of an interpolation expression.
 * @return {?} The length of the interpolation expression in number of characters or `undefined` if none was found.
 */
function defaultTranslationInterpolationExpressionMatcher(translationValue, offset) {
    if (!translationValue.startsWith('{{', offset)) {
        return undefined;
    }
    /** @type {?} */
    const expressionEnd = translationValue.indexOf('}}', offset);
    return expressionEnd >= 2 ? expressionEnd + 2 - offset : undefined;
}
/**
 * Factory function for the `defaultTranslationInterpolationExpressionMatcher`. Used for the `TRANSLATION_INTERPOLATION_EXPRESSION_MATCHER`
 * injection token.
 * @return {?}
 */
function defaultTranslationInterpolationExpressionMatcherFactory() {
    return { matchExpression: defaultTranslationInterpolationExpressionMatcher };
}
/**
 * Injection token that is used for defining the provider for the `InterpolationExpressionMatcher`, which is needed by the
 * `StringInterpolationTranspiler`.
 * @type {?}
 */
const TRANSLATION_INTERPOLATION_EXPRESSION_MATCHER = new InjectionToken('TRANSLATION_INTERPOLATION_EXPRESSION_MATCHER', {
    providedIn: 'root',
    factory: defaultTranslationInterpolationExpressionMatcherFactory
});
/**
 * Markup transpiler that supports interpolation expressions, and which creates a renderer that expands the expression to string using the
 * `TranslocoTranspiler`. Since the `TranslocoTranspiler` can be overridden with a custom implementation that uses a different syntax this
 * class uses a (configurable) `InterpolationExpressionMatcher`. This expression matcher can be overridden with a different implementation
 * that supports the syntax of a custom `TranslocoTranspiler`.
 */
class StringInterpolationTranspiler {
    /**
     * Creates a new `StringInterpolationTranspiler` that uses the specified renderer factory, Transloco transpiler and expression matcher.
     * @param {?} rendererFactory
     * @param {?} translocoTranspiler
     * @param {?} expressionMatcher
     */
    constructor(rendererFactory, translocoTranspiler, expressionMatcher) {
        this.rendererFactory = rendererFactory;
        this.translocoTranspiler = translocoTranspiler;
        this.expressionMatcher = expressionMatcher;
    }
    /**
     * \@inheritdoc
     * @param {?} translation
     * @param {?} offset
     * @return {?}
     */
    tokenize(translation, offset) {
        /** @type {?} */
        const expressionLength = this.expressionMatcher.matchExpression(translation, offset);
        if (!expressionLength) {
            return undefined;
        }
        return {
            nextOffset: offset + expressionLength,
            token: new StringInterpolationSegment(translation.substring(offset, offset + expressionLength))
        };
    }
    /**
     * \@inheritdoc
     * @param {?} offset
     * @param {?} __1
     * @return {?}
     */
    transpile(offset, { tokens, translation }) {
        /** @type {?} */
        const nextToken = tokens[offset];
        if (!(nextToken instanceof StringInterpolationSegment)) {
            return undefined;
        }
        const { interpolationExpression } = nextToken;
        return {
            nextOffset: offset + 1,
            renderer: this.rendererFactory.createTextRenderer((/**
             * @param {?} translationParameters
             * @return {?}
             */
            (translationParameters) => this.translocoTranspiler.transpile(interpolationExpression, translationParameters, translation)))
        };
    }
}
StringInterpolationTranspiler.decorators = [
    { type: Injectable, args: [{ providedIn: 'root' },] }
];
/** @nocollapse */
StringInterpolationTranspiler.ctorParameters = () => [
    { type: TranslationMarkupRendererFactory },
    { type: undefined, decorators: [{ type: Inject, args: [TRANSLOCO_TRANSPILER,] }] },
    { type: undefined, decorators: [{ type: Inject, args: [TRANSLATION_INTERPOLATION_EXPRESSION_MATCHER,] }] }
];
/** @nocollapse */ StringInterpolationTranspiler.ɵprov = ɵɵdefineInjectable({ factory: function StringInterpolationTranspiler_Factory() { return new StringInterpolationTranspiler(ɵɵinject(TranslationMarkupRendererFactory), ɵɵinject(TRANSLOCO_TRANSPILER), ɵɵinject(TRANSLATION_INTERPOLATION_EXPRESSION_MATCHER)); }, token: StringInterpolationTranspiler, providedIn: "root" });
if (false) {
    /**
     * Renderer factory which will be used for rendering the text nodes containing the result of interpolation expressions.
     * @type {?}
     * @private
     */
    StringInterpolationTranspiler.prototype.rendererFactory;
    /**
     * The Transloco transpiler that will be used for evaluating interpolation expressions.
     * @type {?}
     * @private
     */
    StringInterpolationTranspiler.prototype.translocoTranspiler;
    /**
     * Matcher that can find interpolation expressions in a translation value which are supported by the Transloco transpiler.
     * @type {?}
     * @private
     */
    StringInterpolationTranspiler.prototype.expressionMatcher;
}
/**
 * Token class for interpolation expressions.
 */
class StringInterpolationSegment {
    /**
     *  Creates a new `StringInterpolationSegment` for the specified expression.
     * @param {?} interpolationExpression
     */
    constructor(interpolationExpression) {
        this.interpolationExpression = interpolationExpression;
    }
}
if (false) {
    /** @type {?} */
    StringInterpolationSegment.prototype.interpolationExpression;
}

/**
 * @fileoverview added by tsickle
 * Generated from: src/transpilers/contextual-link-transpiler-factory.ts
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Injectable factory service that provides convenience functions for easily creating contextual link transpiler instances.
 */
class ContextualLinkTranspilerFactory {
    /**
     * Creates a new `ContextualLinkTranspilerFactory` that uses the specified renderer factory and link renderers for the creation of
     * `ContextualLinkBlockTranspiler` and `ContextualLinkSubstitutionTranspiler` instances.
     * @param {?} rendererFactory
     * @param {?} linkRenderers
     */
    constructor(rendererFactory, 
    /** Set of link renderers that are used to apply the link model from the translation parameter to the anchor element. */
    linkRenderers) {
        this.rendererFactory = rendererFactory;
        this.linkRenderers = !linkRenderers ? [] : asArray(linkRenderers);
    }
    // Unified implementation of the `createBlockTranspiler` function signatures above.
    /**
     * @param {?} parameterKeyOrStartToken
     * @param {?=} endToken
     * @param {?=} resolveLinkSpecification
     * @return {?}
     */
    createBlockTranspiler(parameterKeyOrStartToken, endToken, resolveLinkSpecification) {
        if (endToken === undefined || resolveLinkSpecification === undefined) {
            /** @type {?} */
            const parameterKey = parameterKeyOrStartToken;
            return this.createBlockTranspiler(`[${parameterKey}]`, `[/${parameterKey}]`, { parameterKey });
        }
        /** @type {?} */
        const startToken = parameterKeyOrStartToken;
        return new ContextualLinkBlockTranspiler(startToken, endToken, resolveLinkSpecification, this.rendererFactory, this.linkRenderers);
    }
    // Unified implementation of the `createSubstitutionTranspiler` function signatures above.
    /**
     * @param {?} parameterKeyOrToken
     * @param {?=} options
     * @return {?}
     */
    createSubstitutionTranspiler(parameterKeyOrToken, options) {
        if (options === undefined) {
            return this.createSubstitutionTranspiler(`[${parameterKeyOrToken}]`, {
                label: { resolve: (/**
                     * @param {?} params
                     * @return {?}
                     */
                    (params) => params[parameterKeyOrToken].label) },
                link: { resolve: (/**
                     * @param {?} params
                     * @return {?}
                     */
                    (params) => params[parameterKeyOrToken].link) }
            });
        }
        return new ContextualLinkSubstitutionTranspiler(parameterKeyOrToken, options, this.rendererFactory, this.linkRenderers);
    }
}
ContextualLinkTranspilerFactory.decorators = [
    { type: Injectable, args: [{ providedIn: 'root' },] }
];
/** @nocollapse */
ContextualLinkTranspilerFactory.ctorParameters = () => [
    { type: TranslationMarkupRendererFactory },
    { type: undefined, decorators: [{ type: Inject, args: [LinkRenderer,] }, { type: Optional }] }
];
/** @nocollapse */ ContextualLinkTranspilerFactory.ɵprov = ɵɵdefineInjectable({ factory: function ContextualLinkTranspilerFactory_Factory() { return new ContextualLinkTranspilerFactory(ɵɵinject(TranslationMarkupRendererFactory), ɵɵinject(LinkRenderer, 8)); }, token: ContextualLinkTranspilerFactory, providedIn: "root" });
if (false) {
    /**
     * Set of link renderers that are used to apply the link model from the translation parameter to the anchor element.
     * @type {?}
     * @private
     */
    ContextualLinkTranspilerFactory.prototype.linkRenderers;
    /**
     * Renderer factory used to create the renderer for the HTML anchor element that will be substituted in place of the token.
     * @type {?}
     * @private
     */
    ContextualLinkTranspilerFactory.prototype.rendererFactory;
}

/**
 * @fileoverview added by tsickle
 * Generated from: src/default-link-renderers.ts
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Link renderer for string values, where the value is interpreted as the URL to an external resource. The renderer will set the browser
 * target for the link to `_blank` (a new window/tab).
 */
class StringLinkRenderer extends LinkRenderer {
    /**
     * \@inheritdoc
     * @param {?} link
     * @return {?}
     */
    supports(link) {
        return typeof link === 'string';
    }
    /**
     * \@inheritdoc
     * @param {?} link
     * @param {?} targetElement
     * @return {?}
     */
    render(link, targetElement) {
        targetElement.href = link;
        targetElement.target = '_blank';
    }
}
StringLinkRenderer.decorators = [
    { type: Injectable }
];
/**
 * Link renderer that supports rendering of `ExternalLink` objects.
 */
class ExternalLinkObjectLinkRenderer extends LinkRenderer {
    /**
     * \@inheritdoc
     * @param {?} link
     * @return {?}
     */
    supports(link) {
        return isExternalLinkObject(link);
    }
    /**
     * \@inheritdoc
     * @param {?} link
     * @param {?} targetElement
     * @return {?}
     */
    render(link, targetElement) {
        targetElement.href = link.url;
        if (typeof link.target === 'string') {
            targetElement.target = link.target;
        }
    }
}
ExternalLinkObjectLinkRenderer.decorators = [
    { type: Injectable }
];

/**
 * @fileoverview added by tsickle
 * Generated from: src/translation-markup-transpiler.token.ts
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Token used to define the `TranslationMarkupTranspiler` providers that should be available for injection.
 * @type {?}
 */
const TRANSLATION_MARKUP_TRANSPILER = new InjectionToken('TRANSLATION_MARKUP_TRANSPILER');
/**
 * Creates a provider definition for the specified markup transpiler. The resulting `Provider` object can be used to make the transpiler
 * available within a module or component injector scope.
 *
 * @param {?} transpiler Specification of the markup transpiler for which the provider definition is to be created.
 * @return {?}
 */
function provideTranslationMarkupTranspiler(transpiler) {
    return bindProvider(TRANSLATION_MARKUP_TRANSPILER, transpiler, { multi: true });
}

/**
 * @fileoverview added by tsickle
 * Generated from: src/default-transloco-markup-transpilers.ts
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Factory function for the default set of markup transpilers. Can be included in the providers array of an `\@NgModule` or `\@Component`
 * decorated class.
 *
 * @return {?} A `Provider` array for the default Transloco markup transpilers.
 */
function defaultTranslocoMarkupTranspilers() {
    return [
        provideTranslationMarkupTranspiler(BoldTextTranspiler),
        provideTranslationMarkupTranspiler(ItalicTextTranspiler),
        provideTranslationMarkupTranspiler(LinkTranspiler),
        provideLinkRenderer(StringLinkRenderer),
        provideLinkRenderer(ExternalLinkObjectLinkRenderer)
    ];
}

/**
 * @fileoverview added by tsickle
 * Generated from: src/inherit-transpilers.ts
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Returns a provider that makes transpilers from a parent injector available in the child injector. This is needed if you define
 * additional translation markup transpilers that are scoped to a specific module or component. Doing so will override the transpilers from
 * the parent injector. Including the provider from the `inheritTranslationMarkupTranspilers` function will make those transpilers also
 * available.
 * @return {?}
 */
function inheritTranslationMarkupTranspilers() {
    return {
        provide: TRANSLATION_MARKUP_TRANSPILER,
        useFactory: (/**
         * @param {?} transpilers
         * @return {?}
         */
        (transpilers) => transpilers),
        deps: [[SkipSelf, TRANSLATION_MARKUP_TRANSPILER]],
        multi: true
    };
}

/**
 * @fileoverview added by tsickle
 * Generated from: src/string-interpolation-transpiler.token.ts
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Injection token for defining the provider to resolve the transpiler used for string interpolation expressions.
 * @type {?}
 */
const STRING_INTERPOLATION_TRANSPILER = new InjectionToken('STRING_INTERPOLATION_TRANSPILER', {
    providedIn: 'root',
    factory: (/**
     * @return {?}
     */
    () => inject(StringInterpolationTranspiler))
});

/**
 * @fileoverview added by tsickle
 * Generated from: src/translation-markup-renderer.model.ts
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * Generated from: src/utils/iterable.ts
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Selects the first projected value that matches the specified predicate. In case no matching value can be found, `undefined` will be
 * returned.
 *
 * @template T, U
 * @param {?} iterable Iterable that produces the values that will be projected.
 * @param {?} project  Mapping function that is applied to values from the iterable.
 * @param {?} match    Predicate function that determines whether the projected value should be returned by this function.
 * @return {?} First project value from the iterable that matches the specified predicate or `undefined` no such value was found.
 */
function selectFirstWhere(iterable, project, match) {
    for (const value of iterable) {
        /** @type {?} */
        const projectedValue = project(value);
        if (match(projectedValue)) {
            return projectedValue;
        }
    }
    return undefined;
}

/**
 * @fileoverview added by tsickle
 * Generated from: src/translation-markup-transpiler.model.ts
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Interface for objects that can transpile translation values (including markup tokens) to a translation renderer.
 * @record
 */
function TranslationMarkupTranspiler() { }
if (false) {
    /**
     * Attempts to recognize a token at the specified position for the given translation value. If a supported token is found, then the
     * token is returned in an arbitrary representation that suits the transpiler, including the offset at which the tokenization process
     * should continue. When no supported token is found `undefined` is returned.
     *
     * @param {?} translation Translation value that is to be tokenized.
     * @param {?} offset      Position where the token should be recognized
     * @return {?} An object containing the token and next tokenization offset or `undefined` when no token is recognized.
     */
    TranslationMarkupTranspiler.prototype.tokenize = function (translation, offset) { };
    /**
     * Transpiles the token sequence (from the given context) starting at the specified offset. If the transpiler can parse the token
     * sequence it returns a `TranslationMarkupRenderer` and the offset of the next unparsed token. In case the transpiler is unable to
     * parse the token sequence at the specified offset `undefined` will be returned as result.
     *
     * @param {?} offset  Position within the token sequence where the transpilation should be performed.
     * @param {?} context Context that contains the token sequence that is to be transpiled and recursive transpilation support.
     * @return {?} An object containing the offset of the next unparsed token and the `TranslationMarkupRenderer` that was created, or
     *                  `undefined` if the transpiler is unable to parse the token sequence.
     */
    TranslationMarkupTranspiler.prototype.transpile = function (offset, context) { };
}
/**
 * Description of a successful tokenization result.
 * @record
 */
function TokenizeResult() { }
if (false) {
    /**
     * Offset at which the tokenization process should continue.
     * @type {?}
     */
    TokenizeResult.prototype.nextOffset;
    /**
     * Token that was recognized.
     * @type {?}
     */
    TokenizeResult.prototype.token;
}
/**
 * Description of a successful transpilation result.
 * @record
 */
function TranspileResult() { }
if (false) {
    /**
     * Offset at which the transpilation process should continue.
     * @type {?}
     */
    TranspileResult.prototype.nextOffset;
    /**
     * Renderer that was created as result of the parsed translation value tokens.
     * @type {?}
     */
    TranspileResult.prototype.renderer;
}
/**
 * Outcome of multiple transpile calls executed sequentially over (part of) a token sequence.
 * @record
 */
function SequentialTranspilationResult() { }
if (false) {
    /**
     * Offset at which the transpilation process should continue.
     * @type {?}
     */
    SequentialTranspilationResult.prototype.nextOffset;
    /**
     * Renderers that were created as result of the parsed translation value tokens.
     * @type {?}
     */
    SequentialTranspilationResult.prototype.renderers;
}
/**
 * Context used during the transpilation process.
 */
class TranslationMarkupTranspilerContext {
    /**
     * Creates a new transpiler context for the specified token sequence and transpilers.
     * @param {?} tokens
     * @param {?} translation
     * @param {?} transpilers
     */
    constructor(tokens, translation, transpilers) {
        this.tokens = tokens;
        this.translation = translation;
        this.transpilers = transpilers;
    }
    /**
     * Attempts to transpile the token sequence at the specified offset. This is done by finding the first transpiler from the set of
     * available transpilers that can successfully parse the token sequence.
     *
     * @param {?} offset Position within the token sequence where the transpilation should be performed.
     * @return {?} Transpilation result of the first transpiler in the transpiler set that can parse the token sequence at the specified
     *                 offset. `undefined` is returned if none if the transpilers can transpile the token sequence.
     */
    transpile(offset) {
        return selectFirstWhere(this.transpilers, (/**
         * @param {?} transpiler
         * @return {?}
         */
        (transpiler) => transpiler.transpile(offset, this)), (/**
         * @param {?} result
         * @return {?}
         */
        (result) => result !== undefined && result.nextOffset !== offset));
    }
    /**
     * Starts a sequential transpilation process that continues transpiling the token sequence until the end has been reached or when the
     * given `stopTranspiling` function returns `true`. The result is an object that contains the offset where the transpilation ended and
     * an array of the markup renderer functions that were created by the transpilers.
     *
     * @param {?} startOffset     Position within the token sequence where the transpilation process should be started.
     * @param {?} stopTranspiling Function that tells whether the sequential transpilation should stop at the specified offet and token at
     *                          that offset in the token sequence. The function must return `true` if the transpilation should stop.
     * @return {?} An object that describes the result of the sequential transpilation process: an array of the created
     *                          translation markup renderers (`renderers` property) and the offset where the process stopped (`nextOffset`).
     */
    transpileUntil(startOffset, stopTranspiling) {
        /** @type {?} */
        let offset = startOffset;
        /** @type {?} */
        const renderers = [];
        while (offset < this.tokens.length && !stopTranspiling(this.tokens[offset], offset)) {
            /** @type {?} */
            const transpileResult = this.transpile(offset);
            if (transpileResult) {
                renderers.push(transpileResult.renderer);
                offset = transpileResult.nextOffset;
            }
            else {
                offset++;
            }
        }
        return {
            renderers,
            nextOffset: offset
        };
    }
}
if (false) {
    /**
     * Sequence of tokens representing the translation value that should be transpiled.
     * @type {?}
     */
    TranslationMarkupTranspilerContext.prototype.tokens;
    /**
     * Translation dictionary containing the translation that is transpiled.
     * @type {?}
     */
    TranslationMarkupTranspilerContext.prototype.translation;
    /**
     * Transpilers which are to be used to parse and convert the token sequence.
     * @type {?}
     * @private
     */
    TranslationMarkupTranspilerContext.prototype.transpilers;
}

/**
 * @fileoverview added by tsickle
 * Generated from: src/transpilers/string-literal-transpiler.ts
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Transpiler used to capture literal translation texts. This transpiler will always be used as the last transpiler during the tokenization
 * and transpilation process, so it can catch any character that would otherwise not be transpiled.
 */
class StringLiteralTranspiler {
    /**
     * Creates a new `StringLiteralTranspiler` that uses the specified renderer factory.
     * @param {?} rendererFactory
     */
    constructor(rendererFactory) {
        this.rendererFactory = rendererFactory;
    }
    /**
     * \@inheritdoc
     * @param {?} translation
     * @param {?} offset
     * @return {?}
     */
    tokenize(translation, offset) {
        return {
            nextOffset: offset + 1,
            token: translation.charAt(offset)
        };
    }
    /**
     * \@inheritdoc
     * @param {?} start
     * @param {?} __1
     * @return {?}
     */
    transpile(start, { tokens }) {
        /** @type {?} */
        let end = start;
        while (end < tokens.length && typeof tokens[end] === 'string') {
            end++;
        }
        if (end === start) {
            return undefined;
        }
        /** @type {?} */
        const stringLiteral = tokens.slice(start, end).join('');
        return {
            nextOffset: end,
            renderer: this.rendererFactory.createTextRenderer(stringLiteral)
        };
    }
}
StringLiteralTranspiler.decorators = [
    { type: Injectable, args: [{ providedIn: 'root' },] }
];
/** @nocollapse */
StringLiteralTranspiler.ctorParameters = () => [
    { type: TranslationMarkupRendererFactory }
];
/** @nocollapse */ StringLiteralTranspiler.ɵprov = ɵɵdefineInjectable({ factory: function StringLiteralTranspiler_Factory() { return new StringLiteralTranspiler(ɵɵinject(TranslationMarkupRendererFactory)); }, token: StringLiteralTranspiler, providedIn: "root" });
if (false) {
    /**
     * Renderer factory used for creating the renderer that renders the literal string text in a text node.
     * @type {?}
     * @private
     */
    StringLiteralTranspiler.prototype.rendererFactory;
}

/**
 * @fileoverview added by tsickle
 * Generated from: src/utils/observe-property.ts
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Observes the specified property and returns a stream that emits all values which are assigned to the property. When subscribing to the
 * resulting stream it will always first emit the current value of the property, followed by all new values that are assigned to it.
 *
 * @template T, K
 * @param {?} target Object containing the property.
 * @param {?} key    Key of the property that is to be observed.
 * @return {?} A stream of all values that are assigned to the specified property, starting with the current value of the property.
 */
function observeProperty(target, key) {
    /**
     * @record
     */
    function GetAccessorWithValueStream() { }
    if (false) {
        /** @type {?|undefined} */
        GetAccessorWithValueStream.prototype.__value$;
        /* Skipping unhandled member: (): T[K];*/
    }
    /** @type {?} */
    const propertyDescriptor = getPropertyDescriptor(target, key);
    /** @type {?} */
    const originalGetAccessor = propertyDescriptor && propertyDescriptor.get;
    // If the specified property is already observed return the value stream that was previously created for this property.
    if (originalGetAccessor && originalGetAccessor.__value$) {
        return originalGetAccessor.__value$;
    }
    /** @type {?} */
    const originalSetAccessor = propertyDescriptor && propertyDescriptor.set;
    // tslint:disable-line:no-unbound-method
    /** @type {?} */
    const subject = new BehaviorSubject(target[key]);
    /** @type {?} */
    const value$ = subject.asObservable();
    /** @type {?} */
    const newGetAccessor = originalGetAccessor
        ? (/**
         * @return {?}
         */
        () => originalGetAccessor.call(target))
        : (/**
         * @return {?}
         */
        () => subject.getValue());
    newGetAccessor.__value$ = value$;
    Object.defineProperty(target, key, {
        get: newGetAccessor,
        /**
         * @param {?} newValue
         * @return {?}
         */
        set(newValue) {
            if (originalSetAccessor !== undefined) {
                originalSetAccessor.call(target, newValue);
            }
            /** @type {?} */
            const nextValue = originalGetAccessor ? originalGetAccessor.call(target) : newValue;
            if (nextValue !== subject.getValue()) {
                subject.next(nextValue);
            }
        }
    });
    return value$;
}
/**
 * Finds the property descriptor for the property `key` in the specified target. In contrast to `Object.getOwnPropertyDescriptor` this
 * function will descent the prototype hierarchy of the specified target to find the property descriptor, making it suitable for properties
 * that are define through accessor functions on classes.
 *
 * @param {?} target Object that containing the property.
 * @param {?} key    Key of the property whose descriptor is to be retrieved.
 * @return {?} The descriptor for the specified property or `undefined` if no such property exists on the target object.
 */
function getPropertyDescriptor(target, key) {
    if (target === null || target === undefined) {
        return undefined;
    }
    /** @type {?} */
    const descriptor = Object.getOwnPropertyDescriptor(target, key);
    return descriptor !== undefined ? descriptor : getPropertyDescriptor(Object.getPrototypeOf(target), key);
}

/**
 * @fileoverview added by tsickle
 * Generated from: src/utils/predicates.ts
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Checks whether the specified value is not exactly equal to `undefined`.
 *
 * @template T
 * @param {?} value Value which is to be checked.
 * @return {?} `true` if the specified value is not exactly equal to `undefined` or `true` if it is.
 */
function notUndefined(value) {
    return value !== undefined;
}

/**
 * @fileoverview added by tsickle
 * Generated from: src/create-translation-markup-renderer.ts
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Creates a translation markup rendering function for the specified translation value.
 *
 * @param {?} translationValue Translation value containing markup tokens that is to be converted to a rendering function.
 * @param {?} transpilers      Ordered set of transpiler that are available for the conversion.
 * @param {?} translation      Translation dictionary containing the translation value.
 * @return {?} A rendering function that can render the translation given translation parameter to a specified target element.
 */
function createTranslationMarkupRenderer(translationValue, transpilers, translation) {
    /** @type {?} */
    const tokens = tokenize(translationValue, transpilers);
    /** @type {?} */
    const renderers = transpile(tokens, transpilers, translation);
    return (/**
     * @param {?} target
     * @param {?} translationParams
     * @return {?}
     */
    function renderTranslation(target, translationParams) {
        while (target.lastChild) {
            target.removeChild(target.lastChild);
        }
        for (const render of renderers) {
            target.appendChild(render(translationParams));
        }
    });
}
/**
 * Converts the given translation value into a sequence of tokens that can be parsed by the specified transpilers. Note that the order of
 * the transpilers is used to resolve conflicts when two or more transpilers can recognize a token at a specific location within the
 * translation value. Tokens are generated based on the first transpiler that is able to create token at a certain offset.
 *
 * @param {?} translationValue Translation value that is to be tokenized.
 * @param {?} transpilers      Ordered set transpilers used to convert the translation value into tokens.
 * @return {?} A sequence of tokens which can be parsed by the given transpilers and converted into markup renderers.
 */
function tokenize(translationValue, transpilers) {
    /** @type {?} */
    const tokens = [];
    /** @type {?} */
    let offset = 0;
    while (offset < translationValue.length) {
        /** @type {?} */
        const tokenizeResult = selectFirstWhere(transpilers, (/**
         * @param {?} transpiler
         * @return {?}
         */
        (transpiler) => transpiler.tokenize(translationValue, offset)), notUndefined);
        if (tokenizeResult) {
            offset = tokenizeResult.nextOffset;
            tokens.push(tokenizeResult.token);
        }
        else {
            offset++;
        }
    }
    return tokens;
}
/**
 * Transpiles the given sequence of tokens using the provded transpilers into a `TranslationMarkupRenderer`.
 *
 * @param {?} tokens      Token sequence that is to be transpiled.
 * @param {?} transpilers Transpilers which are to be used to parse and convert the token sequence.
 * @param {?} translation Translation dictionary containing the translation value that is represented by the given tokens.
 * @return {?} A `TranslationMarkupRenderer` that was constructed by the transpilers from the token sequence.
 */
function transpile(tokens, transpilers, translation) {
    /** @type {?} */
    const transpilerContext = new TranslationMarkupTranspilerContext(tokens, translation, transpilers);
    return transpilerContext.transpileUntil(0, (/**
     * @return {?}
     */
    () => false)).renderers;
}

/**
 * @fileoverview added by tsickle
 * Generated from: src/transloco-markup.component.ts
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
// tslint:disable:component-selector use-component-view-encapsulation no-input-rename
/**
 * Component that supports rendering of translations including HTML markup.
 */
class TranslocoMarkupComponent {
    /**
     * Creates a new `TranslocoMarkupComponent` instance.
     * @param {?} hostElement
     * @param {?} translocoService
     * @param {?} missingHandler
     * @param {?} providedScope
     * @param {?} providedLanguage
     * @param {?} providedTranspilers
     * @param {?} stringInterpolationTranspiler
     * @param {?} stringLiteralTranspiler
     */
    constructor(hostElement, translocoService, missingHandler, providedScope, providedLanguage, providedTranspilers, stringInterpolationTranspiler, stringLiteralTranspiler) {
        this.hostElement = hostElement;
        this.translocoService = translocoService;
        this.missingHandler = missingHandler;
        this.providedScope = providedScope;
        this.providedLanguage = providedLanguage;
        this.providedTranspilers = providedTranspilers;
        this.stringInterpolationTranspiler = stringInterpolationTranspiler;
        this.stringLiteralTranspiler = stringLiteralTranspiler;
        /**
         * Root subscription to which all subscriptions are added that depend on the life cycle of the component instance.
         */
        this.subscriptions = new Subscription();
    }
    /**
     * Initializes the component. When this method is called it starts listening for input property changes on the component and language
     * changes from the `TranslocoService`. This results in a stream containing the rendering function, which is executed for every change
     * to render the translation text including markup to the host element of the component instance.
     * @return {?}
     */
    ngOnInit() {
        /** @type {?} */
        const translationKey$ = observeProperty((/** @type {?} */ (this)), 'translationKey');
        /** @type {?} */
        const translationParameters$ = observeProperty((/** @type {?} */ (this)), 'translationParameters');
        /** @type {?} */
        const inlineLanguage$ = observeProperty((/** @type {?} */ (this)), 'inlineLanguage');
        /** @type {?} */
        const inlineScope$ = observeProperty((/** @type {?} */ (this)), 'inlineScope');
        /** @type {?} */
        const inlineTranspilers$ = observeProperty((/** @type {?} */ (this)), 'inlineTranspilers');
        /** @type {?} */
        const mergeTranspilers$ = observeProperty((/** @type {?} */ (this)), 'mergeTranspilers').pipe(map((/**
         * @param {?} mergeTranspilers
         * @return {?}
         */
        (mergeTranspilers) => mergeTranspilers !== false)));
        // Create the language$ stream that defines which in what language the translation text should be displayed.
        /** @type {?} */
        const langChanges$ = this.translocoService.langChanges$;
        /** @type {?} */
        const activeLanguage$ = this.translocoService.config.reRenderOnLangChange ? langChanges$ : langChanges$.pipe(first(), shareReplay({ bufferSize: 1, refCount: true }));
        /** @type {?} */
        const language$ = resolveLanguage(inlineLanguage$, of(this.providedLanguage || undefined), activeLanguage$);
        // Create the scope$ stream that emits an array of scopes to be used for resolving the translation text.
        /** @type {?} */
        const scopes$ = inlineScope$.pipe(map((/**
         * @param {?} inlineScope
         * @return {?}
         */
        (inlineScope) => inlineScope || this.providedScope)), distinctUntilChanged(), map((/**
         * @param {?} scope
         * @return {?}
         */
        (scope) => Array.isArray(scope) ? scope : [!!scope ? scope : undefined])));
        // Using the language$ and scope$ stream obtain the translation$ stream that emits the translation dictionary to use.
        /** @type {?} */
        const translation$ = combineLatest([language$, scopes$]).pipe(switchMap((/**
         * @param {?} __0
         * @return {?}
         */
        ([language, scopes]) => combineLatest(scopes.map((/**
         * @param {?} scope
         * @return {?}
         */
        (scope) => {
            /** @type {?} */
            const scopeName = getScopeName(scope);
            /** @type {?} */
            const path = getLanguagePath(language, scopeName);
            /** @type {?} */
            const inlineLoader = getScopeInlineLoader(scope);
            return this.translocoService._loadDependencies(path, inlineLoader);
        }))).pipe(map((/**
         * @return {?}
         */
        () => this.translocoService.getTranslation(language)))))));
        // Define the translationValue$ stream that emits the (unparsed) translation text that is to be displayed.
        /** @type {?} */
        const translationValue$ = combineLatest([translationKey$, translation$]).pipe(map((/**
         * @param {?} __0
         * @return {?}
         */
        ([key, translation]) => {
            if (key === undefined) {
                return { value: '', translation };
            }
            /** @type {?} */
            const useFallbackTranslation = (/** @type {?} */ (this.translocoService.config.missingHandler)).useFallbackTranslation || false;
            /** @type {?} */
            const firstFallbackLanguage = asArray(this.translocoService.config.fallbackLang || [])[0];
            /** @type {?} */
            const fallbackTranslation = useFallbackTranslation && firstFallbackLanguage ? [this.translocoService.getTranslation(firstFallbackLanguage)] : [];
            return this.getTranslationValue(key, [translation, ...fallbackTranslation]);
        })));
        // Create the transpilers$ stream based on the inline providers, the provided providers and the merge transpilers setting.
        /** @type {?} */
        const transpilers$ = combineLatest([inlineTranspilers$, mergeTranspilers$]).pipe(map((/**
         * @param {?} __0
         * @return {?}
         */
        ([inlineTranspilers, mergeTranspilers]) => [
            ...(inlineTranspilers ? asArray(inlineTranspilers) : []),
            ...(this.providedTranspilers && (!inlineTranspilers || mergeTranspilers) ? asFlatArray(this.providedTranspilers) : []),
            this.stringInterpolationTranspiler,
            this.stringLiteralTranspiler
        ])));
        // Finally we can create the render$ stream that emits the rendering function based on the translation text and the tranpilers.
        /** @type {?} */
        const render$ = combineLatest([translationValue$, transpilers$]).pipe(map((/**
         * @param {?} __0
         * @return {?}
         */
        ([{ translation, value }, transpilers]) => createTranslationMarkupRenderer(value, transpilers, translation))));
        // By combining the render$ and translationParameters$ stream the component can render the translation whenever something changes.
        this.subscriptions.add(combineLatest([render$, translationParameters$]).subscribe((/**
         * @param {?} __0
         * @return {?}
         */
        ([render, translationParameters]) => render(this.hostElement.nativeElement, translationParameters || {}))));
    }
    /**
     * Called when the component is destroyed. Will cleanup any active subscriptions.
     * @return {?}
     */
    ngOnDestroy() {
        this.subscriptions.unsubscribe();
    }
    /**
     * Determines the translation value for the specified `key` and `translations`. Will return the first value of a translation which
     * contains the specified key. If no such translation exists, the `TranslocoMissingHandler` is used to obtain a value. For translations
     * where the value is an empty string the `TranslocoMissingHandler` is also used to determine if such values are allowed. If so, the
     * function will also return an empty string as translation value. When empty values are not allowed these will be considered as a
     * missing translation value for the tested translation.
     *
     * @private
     * @param {?} key          Key for which the translation value is to be retrieved.
     * @param {?} translations Set of translations which are to be checked for the translation value identified by `key` parameter.
     * @return {?} An object that defines the translation value and the translation dictionary containing the value.
     */
    getTranslationValue(key, translations) {
        /** @type {?} */
        const allowEmptyValues = (/** @type {?} */ (this.translocoService.config.missingHandler)).allowEmpty;
        // Find the first translation containing the specified translation key and obtain the translation value.
        /** @type {?} */
        const result = selectFirstWhere(translations, (/**
         * @param {?} translation
         * @return {?}
         */
        (translation) => {
            /** @type {?} */
            const value = translation[key];
            if (value === undefined || value === '' && !allowEmptyValues) {
                return undefined;
            }
            return { value: String(value), translation };
        }), notUndefined);
        // Return the result or invoke the missing handler if no translation value was found in the specified translations.
        return result || {
            value: this.missingHandler.handle(key, Object.assign({ activeLang: this.translocoService.getActiveLang() }, this.translocoService.config)),
            translation: translations[0]
        };
    }
}
TranslocoMarkupComponent.decorators = [
    { type: Component, args: [{
                selector: 'transloco',
                template: '',
                changeDetection: ChangeDetectionStrategy.OnPush,
                encapsulation: ViewEncapsulation.None,
                styles: [':host { display: inline; }']
            }] }
];
/** @nocollapse */
TranslocoMarkupComponent.ctorParameters = () => [
    { type: ElementRef },
    { type: TranslocoService },
    { type: undefined, decorators: [{ type: Inject, args: [TRANSLOCO_MISSING_HANDLER,] }] },
    { type: undefined, decorators: [{ type: Optional }, { type: Inject, args: [TRANSLOCO_SCOPE,] }] },
    { type: undefined, decorators: [{ type: Optional }, { type: Inject, args: [TRANSLOCO_LANG,] }] },
    { type: undefined, decorators: [{ type: Optional }, { type: Inject, args: [TRANSLATION_MARKUP_TRANSPILER,] }] },
    { type: undefined, decorators: [{ type: Inject, args: [STRING_INTERPOLATION_TRANSPILER,] }] },
    { type: StringLiteralTranspiler }
];
TranslocoMarkupComponent.propDecorators = {
    translationKey: [{ type: Input, args: ['key',] }],
    translationParameters: [{ type: Input, args: ['params',] }],
    inlineLanguage: [{ type: Input, args: ['lang',] }],
    inlineScope: [{ type: Input, args: ['scope',] }],
    inlineTranspilers: [{ type: Input, args: ['transpilers',] }],
    mergeTranspilers: [{ type: Input }]
};
if (false) {
    /**
     * Key of the translation text that needs to be displayed. If no value is specified, the component will render an empty element.
     * @type {?}
     */
    TranslocoMarkupComponent.prototype.translationKey;
    /**
     * Optional object that contains the values to be used for parameterized translation texts.
     * @type {?}
     */
    TranslocoMarkupComponent.prototype.translationParameters;
    /**
     * Language in which the text is the displayed. Overrides the provided language and language set using the translation service.
     * @type {?}
     */
    TranslocoMarkupComponent.prototype.inlineLanguage;
    /**
     * Optional translation scope.
     * @type {?}
     */
    TranslocoMarkupComponent.prototype.inlineScope;
    /**
     * Transpilers used to parse and render translation texts with markup. Merged with the provided transpilers.
     * @type {?}
     */
    TranslocoMarkupComponent.prototype.inlineTranspilers;
    /**
     * Whether the inline transpilers should be merged with the provided ones. If set to `false` only the inline transpilers are used.
     * @type {?}
     */
    TranslocoMarkupComponent.prototype.mergeTranspilers;
    /**
     * Root subscription to which all subscriptions are added that depend on the life cycle of the component instance.
     * @type {?}
     * @private
     */
    TranslocoMarkupComponent.prototype.subscriptions;
    /**
     * Reference to the element which serves as the host for the component instance.
     * @type {?}
     * @private
     */
    TranslocoMarkupComponent.prototype.hostElement;
    /**
     * `TranslocoService` instance that is to be used for loading translations and obtaining the Transloco configuration.
     * @type {?}
     * @private
     */
    TranslocoMarkupComponent.prototype.translocoService;
    /**
     * Handler used for translation keys for which no translation value is defined in the active language.
     * @type {?}
     * @private
     */
    TranslocoMarkupComponent.prototype.missingHandler;
    /**
     * Translation scope that is provided via the module/component injectors.
     * @type {?}
     * @private
     */
    TranslocoMarkupComponent.prototype.providedScope;
    /**
     * Language that is provided via the module/component injectors.
     * @type {?}
     * @private
     */
    TranslocoMarkupComponent.prototype.providedLanguage;
    /**
     * Markup transpilers provided via the module/component injectors.
     * @type {?}
     * @private
     */
    TranslocoMarkupComponent.prototype.providedTranspilers;
    /**
     * Transpiler that is used for expanding string interpolation expressions.
     * @type {?}
     * @private
     */
    TranslocoMarkupComponent.prototype.stringInterpolationTranspiler;
    /**
     * Transpiler used for rendering all unprocessed tokens in the translation values as literal text.
     * @type {?}
     * @private
     */
    TranslocoMarkupComponent.prototype.stringLiteralTranspiler;
}
/**
 * Determines which language should be active given a sequence of streaming language specifiers. The active languages is resolved based on
 * the order of the language specifiers where the first non `undefined` language is emitted. If that language specifier does not specify a
 * static language (e.g. `'en|static'`), any changes from the downstream language specifiers will be allowed to 'override' the active
 * language.
 *
 * @param {...?} languageSpecifiers An array of language specifier streams.
 * @return {?} An observable that emits the active language based in the language specifiers.
 */
function resolveLanguage(...languageSpecifiers) {
    return languageSpecifiers.reduceRight((/**
     * @param {?} downstreamLanguage$
     * @param {?} languageSpecifier$
     * @return {?}
     */
    (downstreamLanguage$, languageSpecifier$) => languageSpecifier$.pipe(switchMap((/**
     * @param {?} languageSpecifier
     * @return {?}
     */
    (languageSpecifier) => {
        if (languageSpecifier === undefined) {
            return downstreamLanguage$;
        }
        const [isStatic, language] = getPipeValue(languageSpecifier, 'static');
        return concat(of(language), isStatic ? EMPTY : downstreamLanguage$.pipe(skip(1)));
    })))), of('')).pipe(distinctUntilChanged());
}
/**
 * Retrieves the name of the specified Transloco scope.
 *
 * @param {?} scope Scope for which the name is to be returned.
 * @return {?} Name of the specified scope or `undefined` if the scope itself is `undefined`.
 */
function getScopeName(scope) {
    return typeof scope === 'object' ? scope.scope : scope;
}
/**
 * Determines the language path for the specified language and (optional) scope.
 *
 * @param {?} language Language identifier.
 * @param {?=} scope    Scope name (optional).
 * @return {?} Language path.
 */
function getLanguagePath(language, scope) {
    return scope ? `${scope}/${language}` : language;
}
/**
 * Determines the `InlineLoader` for the specified scope (if it is defined).
 *
 * @param {?} scope Scope for which the `InlineLoader` is to be determined.
 * @return {?} Inline loader for the specified scope or `undefined` if the scope does not define one.
 */
function getScopeInlineLoader(scope) {
    return typeof scope === 'object' ? scope.loader : undefined;
}

/**
 * @fileoverview added by tsickle
 * Generated from: src/transloco-markup.module.ts
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Module which exports the `<transloco>` markup component.
 */
class TranslocoMarkupModule {
}
TranslocoMarkupModule.decorators = [
    { type: NgModule, args: [{
                declarations: [TranslocoMarkupComponent],
                exports: [TranslocoMarkupComponent]
            },] }
];

/**
 * @fileoverview added by tsickle
 * Generated from: src/public-api.ts
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * Generated from: public-api.ts
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * Generated from: ngx-transloco-markup.ts
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { BlockTranspiler, BoldTextTranspiler, ContextualLinkBlockTranspiler, ContextualLinkSubstitutionTranspiler, ContextualLinkTranspilerFactory, ItalicTextTranspiler, LinkRenderer, LinkTranspiler, STRING_INTERPOLATION_TRANSPILER, SubstitutionTranspiler, TRANSLATION_INTERPOLATION_EXPRESSION_MATCHER, TRANSLATION_MARKUP_TRANSPILER, TranslationMarkupRendererFactory, TranslationMarkupTranspilerContext, TranslocoMarkupComponent, TranslocoMarkupModule, defaultTranslocoMarkupTranspilers, inheritTranslationMarkupTranspilers, isExternalLinkObject, provideLinkRenderer, provideTranslationMarkupTranspiler, defaultTranslationInterpolationExpressionMatcher as ɵa, defaultTranslationInterpolationExpressionMatcherFactory as ɵb, bindProvider as ɵd, StringLinkRenderer as ɵe, ExternalLinkObjectLinkRenderer as ɵf, StringLiteralTranspiler as ɵh };
//# sourceMappingURL=ngx-transloco-markup.js.map
